"use client";
import { useRef, useState } from "react";
import { Box, Button, Flex } from "@chakra-ui/react";
import {
  FileWrapper,
  InputWrapper,
  SelectWrapper,
} from "@/components/fields/Fields";
import * as Yup from "yup";
import FormComponent from "@/components/form/Form";
import useShowToast from "@/app/hooks/useShowToast";
import { useParams, useRouter } from "next/navigation";
import { createBatch } from "@/app/lib/batches/action";
import MultiSelectWrapper from "@/components/fields/MultiSelectWrapper";
import { Batch } from "../variables/batch";
import { usersRequestList } from "@/app/lib/users/action";
import { prepareUsersDropdown } from "@/utils/common";

interface BatchFormProps {
  usersDropdown: { value: string | number; label: string }[];
  ibcDropdown: { value: string; label: string }[];
  assignPriorityOption: { value: string; label: string }[];
  batch?: Batch;
  isEdit?: boolean;
}

interface FormData {
  name: string;
  file: File[];
  users: string[];
  due_date: string;
  assignment_priority_id: string;
}

const validationSchema = Yup.object({
  name: Yup.string().required("Name is required"),
  file: Yup.array().min(1, "File is required").required("File is required"),
  users: Yup.array()
    .min(1, "User list is required")
    .required("User list is required"),
  due_date: Yup.string().required("Due date is required"),
});

export default function BatchForm({
  usersDropdown,
  ibcDropdown,
  assignPriorityOption,
  batch,
  isEdit,
}: BatchFormProps) {
  const router = useRouter();
  const formRef = useRef(null);
  const showToast = useShowToast();
  const [users, setUsers] = useState(usersDropdown);

  const initialValues = {
    name: batch?.name ?? "",
    file: "",
    users: batch?.users.map((user) => user.id) ?? [],
    due_date: batch?.due_date ?? "",
    assignment_priority_id: batch?.assignment_priority_id ?? "",
  };

  const handleCreate = async (values: FormData) => {
    try {
      const formData = new FormData();
      formData.append("name", values.name);
      formData.append("status", "1");
      values.users.forEach((userId) => {
        formData.append("user_ids[]", userId);
      });
      formData.append("file", values.file[0]);
      formData.append("due_date", values.due_date);
      formData.append("assignment_priority_id", values.assignment_priority_id);
      const response = await createBatch(formData);
      showToast("Batch Created!", response.message, "success");
      router.push("/admin/batches");
    } catch (error: any) {
      showToast("Failed to create", error.message, "error");
    }
  };

  const handleUpdate = async (values: FormData) => {
    try {
      showToast("Batch Updated!", "Batch updated successfully", "success");
      router.push("/admin/batches");
    } catch (error: any) {
      showToast("Failed to update", error.message, "error");
    }
  };

  const handleSubmit = (values: FormData) => {
    if (isEdit) {
      handleUpdate(values);
    } else {
      handleCreate(values);
    }
  };

  const handleSubmitClick = () => {
    formRef?.current?.externalHandleSubmit();
  };

  const handleIbcChange = async (value: string) => {
    const data = await usersRequestList({ ibc_ids: value, pagination: "false" });
    const updatedUsersDropdown = prepareUsersDropdown(data);
    setUsers(updatedUsersDropdown);
  };

  return (
    <Box pt={{ base: "130px", md: "80px", xl: "80px" }} mb="20px">
      <FormComponent
        ref={formRef}
        submitForm={handleSubmit}
        initialValues={initialValues}
        validationSchema={validationSchema}
      >
        <InputWrapper
          id="name"
          type="text"
          fieldName="name"
          label="Batch Name"
          placeholder="Enter Batch Name"
          extra="*"
          defaultValue={initialValues.name}
        />

        <SelectWrapper
          form={false}
          fieldName="ibc_id"
          label="IBC"
          id="ibc_id"
          variant="auth"
          w="full"
          mb="2px"
          fontSize="sm"
          fontWeight="500"
          options={ibcDropdown}
          updateOnParent={(value) => handleIbcChange(String(value))}
        />

        <MultiSelectWrapper
          options={users}
          id="users"
          fieldName="users"
          label="Users"
          placeholder="Select users"
          extra="*"
          defaultValue={usersDropdown.filter((user) =>
            initialValues.users.includes(Number(user.value))
          )}
        />

        <SelectWrapper
          form
          fieldName="assignment_priority_id"
          label="Assignment Priority"
          extra="*"
          defaultValue={initialValues.assignment_priority_id}
          id="assignment_priority_id"
          variant="auth"
          w="full"
          mb="2px"
          fontSize="sm"
          fontWeight="500"
          options={assignPriorityOption}
        />

        <InputWrapper
          id="date"
          type="date"
          fieldName="due_date"
          label="Due date"
          placeholder="Enter due date"
          extra="*"
          defaultValue={initialValues.due_date}
        />

        <FileWrapper
          id="file"
          label="CSV File"
          fieldName="file"
          extra="*"
          accept=".csv"
          subText="CSV up to 2MB"
          multiple={false}
        />

        <Flex w={"full"} justifyContent={"flex-end"} pt={4}>
          <Button
            variant="brand"
            onClick={handleSubmitClick}
            mr={3}
            w={"80px"}
            justifySelf={"flex-end"}
          >
            {isEdit ? "Save" : "Create"}
          </Button>
        </Flex>
      </FormComponent>
    </Box>
  );
}
